Name: zz_local_dns_cache
Summary: This package configures a host do local dns caching with bind
Version: 3
Release: 0.1
License: GPL
Group: Fermi
URL: http://fermilinux.fnal.gov/
Vendor: Fermilab
Packager: Pat Riehecky
BuildRequires: bash tar gzip findutils
Requires: bind bind-chroot
Requires: coreutils policycoreutils selinux-policy-targeted redhat-lsb gawk chkconfig sed bash findutils
Obsoletes: caching-nameserver
Source0: zz_local_dns_cache.tar.gz
BuildRoot: %{_tmppath}/%{name}-%{version}
BuildArch: noarch

%description
Installing this package will reconfigure named to run as a caching nameserver which will only allow connections from localhost. Your /etc/resolv.conf will be modified to call on localhost as primary nameserver.  The dhclient process will be configured to use this nameserver, if present.  Network Manager will be configured to use this nameserver, if present.

You should consider customizing the DNS server search list provided.  When removed you will be restored to using 131.225.0.254 and 8.8.8.8 as SLF is used both on and off site.  If this is not what you want, you will need to customize these.

This package should work on SLF5 and SLF6
%prep
%setup

%build

%install
mkdir -p %{buildroot}/var/named/chroot/etc/

find %{buildroot}/ -type f -exec chmod 644 {} \;
find %{buildroot}/ -type d -exec chmod 750 {} \;
install -m 640 zz_local_dns_cache.conf %{buildroot}/var/named/chroot/etc/

%files
/var/named/chroot/etc/zz_local_dns_cache.conf
%doc zz_local_dns_cache.conf
%doc zz_local_dns_cache-dhclient.conf
%doc named.rfc1912.zones
%doc named.empty
%doc named.localhost
%doc named.loopback
%doc README
%doc README.selinux

%clean
rm -rf %{buildroot}


###########################################################################

%post
# most of these operations are '&' so that the kernel elevator can optimize
# them as much of this IO happens over a limited disk range, this should
# help reduce the time spent on disk activites

# cleanup any running named
service named stop >/dev/null 2>&1

# I don't know why or how, but something keeps removing the named group
# from these files
chown root:named /var/named/chroot/etc/zz_local_dns_cache.conf  >/dev/null 2>&1   &
chown root:named /usr/share/doc/%name-%version/*                >/dev/null 2>&1   &

# make sure my selinux context is right
chcon -t named_conf_t /var/named/chroot/etc/zz_local_dns_cache.conf >/dev/null 2>&1   &
chcon -u system_u /var/named/chroot/etc/zz_local_dns_cache.conf     >/dev/null 2>&1   &
chcon -r object_r /var/named/chroot/etc/zz_local_dns_cache.conf     >/dev/null 2>&1   &

# don't know if this is running or installed, but in a second we are
# going to turn on bind so this can't be running at that time or weirdness
service dnsmasq stop >/dev/null 2>&1  &

# make sure named cannot write to master zones, we are in cache mode only!
setsebool -P named_write_master_zones 0 >/dev/null 2>&1  &

# link the config file from the chroot, so its easy to find
if [[ ! -f /etc/zz_local_dns_cache.conf ]]; then
    ln -s /var/named/chroot/etc/zz_local_dns_cache.conf /etc/zz_local_dns_cache.conf  &
fi

# make the new config file the named.conf master
if [[ -e /var/named/chroot/etc/named.conf ]]; then
    if [[ -L /var/named/chroot/etc/named.conf ]] ; then
        rm -f /var/named/chroot/etc/named.conf
    else
        mv -f /var/named/chroot/etc/named.conf /var/named/chroot/etc/named.conf.rpmbak
    fi
fi
cd /var/named/chroot/etc/
ln -s zz_local_dns_cache.conf named.conf

# make sure my selinux context is right
chcon -h -t named_conf_t /var/named/chroot/etc/named.conf >/dev/null 2>&1   &
chcon -h -u system_u /var/named/chroot/etc/named.conf     >/dev/null 2>&1   &
chcon -h -r object_r /var/named/chroot/etc/named.conf     >/dev/null 2>&1   &

# make sure we have the needed rfc zones in the chroot
# different between 5 and 6, using the 6 config as its fine on 5
if [[ ! -f /var/named/chroot/etc/named.rfc1912.zones ]]; then
    cp -p /usr/share/doc/%name-%version/named.rfc1912.zones /var/named/chroot/etc/
    chmod 640 /var/named/chroot/etc/named.rfc1912.zones
    chown root:named /var/named/chroot/etc/named.rfc1912.zones
fi

# make sure my selinux context is right
chcon -t named_conf_t /var/named/chroot/etc/named.rfc1912.zones >/dev/null 2>&1   &
chcon -u system_u /var/named/chroot/etc/named.rfc1912.zones     >/dev/null 2>&1   &
chcon -r object_r /var/named/chroot/etc/named.rfc1912.zones     >/dev/null 2>&1   &

if [[ ! -f /etc/named.rfc1912.zones ]]; then
    ln -s /var/named/chroot/etc/named.rfc1912.zones /etc/   &
fi

# 5.x needs these copied in
for stubzone in named.empty named.localhost named.loopback; do
    if [[ ! -e /var/named/chroot/var/named/$stubzone ]]; then
        cp -p /usr/share/doc/%name-%version/$stubzone /var/named/chroot/var/named/
        chown root:named /var/named/chroot/var/named/$stubzone  &
        chmod 640 /var/named/chroot/var/named/$stubzone         &

        # make sure my selinux context is right
        chcon -t named_zone_t /var/named/chroot/var/named/$stubzone >/dev/null 2>&1  &
        chcon -u system_u /var/named/chroot/var/named/$stubzone     >/dev/null 2>&1  &
        chcon -r object_r /var/named/chroot/var/named/$stubzone     >/dev/null 2>&1  &
    fi
done

# start at boot
chkconfig named on >/dev/null 2>&1   &

# let things catch up
wait

# Start our newly configured named
service named restart >/dev/null 2>&1

# this is a fresh install
if [[ $1 -eq 1 ]]; then
    # replace /etc/resolv.conf with references to new name server
    TMP=`mktemp`
    echo "# Generated by zz_local_dns_cache" > $TMP
    grep "search " /etc/resolv.conf >> $TMP
    echo "nameserver 127.0.0.1" >> $TMP
    echo "# lookups forwarded to" >> $TMP
    echo "#nameserver 131.225.0.254" >> $TMP
    echo "#nameserver 8.8.8.8" >> $TMP
    cat $TMP > /etc/resolv.conf
    rm -f $TMP
else
    # this should be present or there is no point in running it
    grep '^nameserver 127.0.0.1' /etc/resolv.conf >/dev/null 2>&1
    if [[ $? -eq 1 ]]; then
        echo "nameserver 127.0.0.1" >> /etc/resolv.conf
    fi
fi

restorecon /etc/resolv.conf >/dev/null 2>&1

###########################################################################

%postun
# if this is an uninstall only (not an upgrade)
if [[ $1 -eq 0 ]]; then

    rm -f /etc/zz_local_dns_cache.conf   &

    # if named is using the provided config, deconfigure named
    if [[ `readlink /var/named/chroot/etc/named.conf | xargs -i basename {}` == "zz_local_dns_cache.conf" ]]; then
        service named stop   >/dev/null 2>&1 
        chkconfig named off  >/dev/null 2>&1    &
        rm -f /var/named/chroot/etc/named.conf  &
    fi

    # replace /etc/resolv.conf with references to working nameservers
    TMP=`mktemp`
    grep "search " /etc/resolv.conf >> $TMP
    echo "nameserver 131.225.0.254" >> $TMP
    echo "nameserver 8.8.8.8" >> $TMP
    cat $TMP > /etc/resolv.conf
    rm -f $TMP    &
    restorecon /etc/resolv.conf >/dev/null 2>&1 &

    # let things catch up
    wait
fi

###########################################################################

%triggerin -- NetworkManager
# this is a fresh install
if [[ $1 -eq 1 ]]; then
    for eth in `ls /etc/sysconfig/network-scripts/ifcfg-* >/dev/null | grep -v ifcfg-lo`; do
        grep 'DNS1=127.0.0.1' $eth >/dev/null 2>&1
        if [[ $? -eq 1 ]]; then
            TMP=`mktemp`
            grep -v "^DNS" $eth |grep -v "PEERDNS=" | sort -u  > $TMP
            grep    "^DNS" $eth | sort -u | xargs -i echo "#{}" >> $TMP
            echo "DNS1=127.0.0.1" >> $TMP
            echo "PEERDNS=no" >> $TMP
            cat $TMP | sort -u > $eth
            rm -f $TMP  &
            restorecon $eth >/dev/null 2>&1 &
        fi
    done
    wait   # let things catch up
fi

%triggerun -- NetworkManager
# if this is an uninstall only (not an upgrade)
if [[ $1 -eq 0 ]]; then
    for eth in `ls /etc/sysconfig/network-scripts/ifcfg-* >/dev/null | grep -v ifcfg-lo`; do
        grep 'DNS1=127.0.0.1' $eth >/dev/null 2>&1
        if [[ $? -eq 0 ]]; then
            TMP=`mktemp`
            grep -v "^DNS" $eth | sort -u > $TMP
            grep    "^DNS" $eth | grep -v 'DNS1=127.0.0.1' | grep -v "PEERDNS=" | sort -u | xargs -i echo "#{}" >> $TMP
            echo "DNS1=131.225.0.254" >> $TMP
            echo "DNS2=8.8.8.8" >> $TMP
            echo "PEERDNS=yes" >> $TMP
            cat $TMP | sort -u > $eth
            rm -f $TMP  &
            restorecon $eth >/dev/null 2>&1 &
        fi
    done
    wait   # let things catch up
fi

###########################################################################

%triggerin -- dhclient
# this is a fresh install
if [[ $1 -eq 1 ]]; then

    SLFVERSION=`lsb_release -r| awk '{print $2}'| cut -d'.' -f1`
    DHCP_CONFIG_PATH=''
    if [[ "$SLFVERSION" == '5' ]]; then
        DHCP_CONFIG_PATH='/etc'
     elif [[ "$SLFVERSION" == '6' ]]; then
        DHCP_CONFIG_PATH='/etc/dhcp'
    fi

    if [[ "x$DHCP_CONFIG_PATH" != 'x' ]]; then
        if [[ -e $DHCP_CONFIG_PATH/dhclient.conf ]]; then
        # does dhclient.conf already have all the lines in it?
        # odds are no, but may as well check first
            diff /usr/share/doc/%name-%version/zz_local_dns_cache-dhclient.conf $DHCP_CONFIG_PATH/dhclient.conf | grep '<' >/dev/null 2>&1
            if [[ $? -ne 1 ]]; then
                cat /usr/share/doc/%name-%version/zz_local_dns_cache-dhclient.conf >> $DHCP_CONFIG_PATH/dhclient.conf
            fi
        else
            cp /usr/share/doc/%name-%version/zz_local_dns_cache-dhclient.conf $DHCP_CONFIG_PATH/dhclient.conf
        fi
    fi
fi

# always a good idea to run restorecon, it wont hurt on missing files
restorecon /etc/dhclient.conf >/dev/null 2>&1       &
restorecon /etc/dhcp/dhclient.conf >/dev/null 2>&1  &

# let things catch up
wait

%triggerun -- dhclient
# if this is an uninstall only (not an upgrade)
if [[ $1 -eq 0 ]]; then

    SLFVERSION=`lsb_release -r| awk '{print $2}'| cut -d'.' -f1`
    DHCP_CONFIG_PATH=''
    if [[ "$SLFVERSION" == '5' ]]; then
        DHCP_CONFIG_PATH='/etc'
     elif [[ "$SLFVERSION" == '6' ]]; then
        DHCP_CONFIG_PATH='/etc/dhcp'
    fi

    if [[ -e $DHCP_CONFIG_PATH/dhclient.conf ]]; then
        mv -f $DHCP_CONFIG_PATH/dhclient.conf $DHCP_CONFIG_PATH/dhclient.conf.rpmsave
    fi
fi

# always a good idea to run restorecon, it wont hurt on missing files
restorecon /etc/dhclient.conf >/dev/null 2>&1       &
restorecon /etc/dhcp/dhclient.conf >/dev/null 2>&1  &

# let things catch up
wait

###########################################################################

%triggerin -- dnsmasq
# this is a fresh install
if [[ $1 -eq 1 ]]; then
    chkconfig dnsmasq off # since we chkconfig'd bind on we should turn this off

    sed -i'.rpmbak' \
        -e "s/#no-dhcp-interface=.*/no-dhcp-interface=lo/"  \
        -e "s/#listen-address=.*/listen-address=127.0.0.1/" \
        -e "s/#no-resolv.*/no-resolv\ndomain=fnal.gov\nserver=131.225.0.254\nserver=8.8.8.8/"   \
    /etc/dnsmasq.conf

    restorecon /etc/dnsmasq.conf >/dev/null 2>&1
fi

%triggerun -- dnsmasq
# if this is an uninstall only (not an upgrade)
if [[ $1 -eq 0 ]]; then
    sed -i'.rpmsave'      \
        -e "s/no-resolv/#no-resolv\nserver=131.225.0.254\nserver=8.8.8.8/"   \
    /etc/dnsmasq.conf

    restorecon /etc/dnsmasq.conf >/dev/null 2>&1
fi

###########################################################################

%changelog
* Thu Dec 8 2011 Pat Riehecky <riehecky@fnal.gov> 3-0.1
- fixed a minor chmod problem
- switched search from /etc/sysconfig/network-scripts/ifcfg-eth* to /etc/sysconfig/network-scripts/ifcfg-*|grep -v ifcfg-lo to account for strangly named cards

* Wed Nov 9 2011 Pat Riehecky <riehecky@fnal.gov> 3-0.0
- Almost totally re-written
- Moved NetworkManager to trigger
- Moved dhclient to trigger
- Added dnsmasq trigger
  this rpm enables bind, but if you want instead dnsmasq it should be configured
- Now requires selinux packages, selinux enforcement is however optional
  more info in README.selinux
- Using links to help make it obvious which file bind is reading
- Modified config to use views to help reduce possible non localhost use in bind
- Switched from a dynamic config to a static one for deploy and copy to 'doc'
  it is configured after the style of SLF5 caching-nameserver but with
  the approach from SLF6 bind.  It ballences security and readibility.
- Obsolete caching-nameserver for 5.x its not in 6.x and integration was messy
- Set sane looking defaults rather than ad-hoc creation of forwarder list
   + 131.225.0.254
   + 8.8.8.8

* Mon May 23 2011 Jason Harrington <jason@fnal.gov> 2.0-2
- Fixed bugs dealing with NetworkManager comments in /etc/resolv.conf
- Added default DNS forwarder when no nameserver list found
- Added eth0-specific domain-name-server prepend for use with NetworkManager
- Added post-uninstall bits to undo dhclient tweaks

* Thu May 19 2011 Jason Harrington <jason@fnal.gov> 2.0-1
- Updated for use with SL(F)6x

*Fri Jan 05 2007 Troy Dawson <dawson@fnal.gov> 1.0-2
- Added post and postun to add 127.0.0.1 to the resolv.conf

*Fri Jan 05 2007 Troy Dawson <dawson@fnal.gov> 1.0-1
- First pass, looks good
